import os
import cv2
import shutil
from tqdm import tqdm


class FolderTools:
    def __init__(self, root_path, save_path):
        self.root_path = root_path
        self.save_path = save_path
        os.makedirs(self.save_path, exist_ok=True)

    def rename_folder_files(self, prefix1, prefix2):
        for f in tqdm(os.listdir(self.root_path)):
            f_p = os.path.join(self.root_path, f)
            aim_p = os.path.join(self.root_path, f.replace(prefix1, prefix2))
            os.rename(f_p, aim_p)

    def extract_imgs_from_folder(self, prefix1):
        for r, d, f in os.walk(self.root_path):
            for ff in f:
                if ff.split('.')[-1] in prefix1:
                    f_p = os.path.join(r, ff)
                    img = cv2.imread(f_p)
                    if img is not None:
                        shutil.copy(f_p, os.path.join(self.save_path, ff))

    def clean_one_folder_from_another(self, ):
        # clean root_p from save_p
        f_list = os.listdir(self.root_path)
        f_list_to_do = os.listdir(self.save_path)

        f_list_remove = list(set(f_list).intersection(set(f_list_to_do)))
        # print(len(f_list_remove))
        for f in tqdm(f_list_remove):
            os.remove(os.path.join(self.root_path, f))


if __name__ == '__main__':
    root_p = '/workspace/84_jiqun/home/user/luanjintai/projects/DeepFaceLab_Linux/素材/迪丽热巴/迪丽热巴_2'
    save_p = '/workspace/data/waifu_anime/idx1_out_2'
    folder_tools = FolderTools(root_p, save_p)
    folder_tools.rename_folder_files('.jpg', '_reba_2.jpg')
    # folder_tools.extract_imgs_from_folder(['jpg', 'png'])
    # folder_tools.clean_one_folder_from_another()



import os
import cv2
import shutil
from tqdm import tqdm


def decode_fourcc(cc):
    return "".join([chr((int(cc) >> 8 * i) & 0xFF) for i in range(4)])


class VideoTools:
    def __init__(self, video_p, verbose=True):
        self.video_path = video_p
        _, self.video_name = os.path.split(self.video_path)
        _, self.extension = os.path.splitext(self.video_name)
        if verbose:
            self.print_video_info()

    def print_video_info(self):

        # 获得视频的格式
        cap = cv2.VideoCapture(self.video_path)
        fourcc = cap.get(cv2.CAP_PROP_FOURCC)
        # 获得码率及尺寸
        fps = cap.get(cv2.CAP_PROP_FPS)
        size = (int(cap.get(cv2.CAP_PROP_FRAME_WIDTH)),
                int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT)))
        cap.release()
        print(
            'video info:\nname: {}\nfourcc: {}\nfps: {}\nsize: {}'.format(self.video_name, decode_fourcc(fourcc), fps,
                                                                          size))

    def video_2_h264(self, inplace=True):
        # ffmpeg -i "xx.avi" -vcodec h264 "xx.mp4"
        if not inplace:
            os.system('ffmpeg -i {} -vcodec h264 {}'.format(self.video_path,
                                                            self.video_path.replace(self.extension, '_h264_out.mp4')))
        else:
            temp_p = self.video_path.replace(self.extension, '_temp_copy.mp4')
            os.system(
                'mv {} {} && ffmpeg -i {} -vcodec h264 {} && rm {}'.format(self.video_path, temp_p, temp_p,
                                                                           self.video_path, temp_p))

    # def frame_2_video(self, fps, img_size, num_frames=394):
    #     # fps = 30  # 帧率
    #     img_array = []
    #     img_width = img_size[0]
    #     img_height = img_size[1]
    #     for i in range(num_frames + 1):
    #         filename = self.root_path + '/' + "%04d" % i + ".png"
    #         # filename = self.root_path + '/' + str(i) + ".png"
    #         img = cv2.imread(filename)
    #
    #         if img is None:
    #             print(filename + " is non-existent!")
    #             continue
    #         img_array.append(img)
    #
    #     out = cv2.VideoWriter(self.root_path + '.avi', cv2.VideoWriter_fourcc(*'DIVX'), fps, (img_width, img_height))
    #
    #     for i in range(len(img_array)):
    #         out.write(img_array[i])
    #     out.release()

    def video_2_frame(self, interval=1, out_path=None):
        if out_path is None:
            save_path = self.video_path.split('.mp4')[0] + '/'
        else:
            save_path = out_path

        is_exists = os.path.exists(save_path)
        if not is_exists:
            os.makedirs(save_path)
            print('path of %s is build' % save_path)
        else:
            shutil.rmtree(save_path)
            os.makedirs(save_path)
            print('path of %s already exist and rebuild' % save_path)

        # 开始读视频
        video_capture = cv2.VideoCapture(self.video_path)
        i = 0
        j = 0

        while True:
            success, frame = video_capture.read()
            i += 1
            # print(frame)
            if not success:
                print('done!')
                break
            if i % interval == 0:
                # 保存图片
                j += 1
                save_name = save_path + str(j) + '_' + str(i) + '.png'
                cv2.imwrite(save_name, frame)
                print('image of %s is saved' % save_name)
        video_capture.release()

    def resize_video(self, inplace=False, out_size=(768, 1024)):

        out_p = self.video_path.replace(self.extension, '_{}x{}.mp4'.format(out_size[0], out_size[1]))
        if inplace:
            os.system('mv {} {}'.format(self.video_path, out_p))
            self.video_path, out_p = out_p, self.video_path

        cap = cv2.VideoCapture(self.video_path)
        fps = cap.get(cv2.CAP_PROP_FPS)

        video_writer = cv2.VideoWriter(out_p, cv2.VideoWriter_fourcc(*'mp4v'), fps, out_size)

        s = True
        while s:
            s, img1 = cap.read()
            img = cv2.resize(img1, out_size, cv2.INTER_LINEAR)
            video_writer.write(img)
        cap.release()

        if inplace:
            os.system('rm {} &'.format(self.video_path))

    # def avi2mp4(self):
    #
    #     # 指定写视频的格式, I420-avi, MJPG-mp4
    #     videoWriter = cv2.VideoWriter(self.root_path.split('.')[0] + '.mp4', cv2.VideoWriter_fourcc(*'mp4v'),
    #                                   fps, size)
    #
    #     # 读帧
    #     success, frame = cap.read()
    #
    #     while success:
    #         # cv2.waitKey(1000 / int(fps))  # 延迟
    #         videoWriter.write(frame)  # 写视频帧
    #         success, frame = cap.read()  # 获取下一帧


if __name__ == '__main__':
    root_p = '/workspace/codes/DeepFaceLab_Real_Time/out_video/pyy-ljt-liae-ud-stage1.avi'
    # root_p = '/workspace/codes/cv_toolbox/tmp'
    save_p = '/workspace/data/mysterybox/mysterybox_origin/210427/out'
    video_tools = VideoTools(root_p, save_p)
    # video_tools.frame_2_video(fps=30, img_size=(1920, 1080), num_frames=1149)
    # video_tools.video_2_frame()
    # video_tools.resize_video()
    video_tools.avi2mp4()



from tqdm import tqdm
import os
from PIL import Image
import numpy as np
import cv2
import imageio


class ImgTools:
    def __init__(self, img_path, root_path, save_path):
        self.img_path = img_path
        self.root_path = root_path
        self.save_path = save_path
        os.makedirs(self.save_path, exist_ok=True)

    def alpha_channel_white(self, ):
        for f in tqdm(os.listdir(self.root_path)):
            f_p = os.path.join(self.root_path, f)
            pic = Image.open(f_p)
            pic = pic.convert('RGBA')  # 转为RGBA模式
            width, height = pic.size
            array = pic.load()  # 获取图片像素操作入口
            for i in range(width):
                for j in range(height):
                    pos = array[i, j]  # 获得某个像素点，格式为(R,G,B,A)元组
                    try:
                        isEdit = (pos[3] == 0)
                        if isEdit:
                            array[i, j] = (255, 255, 255, 0)
                    except IndexError or TypeError:
                        break
            pic.save(os.path.join(self.save_path, f), 'PNG')

    def convert_2_3channel(self, ):
        for f in tqdm(os.listdir(self.root_path)):
            f_p = os.path.join(self.root_path, f)
            img = Image.open(f_p)
            # 将一个4通道转化为rgb三通道
            img = img.convert("RGB")
            img = np.array(img, dtype=np.uint8)[:, :, [2, 1, 0]]
            cv2.imwrite(os.path.join(self.save_path, f), img)

    def resize_floder_img(self, aim_size, channel):
        for f in tqdm(os.listdir(self.root_path)):
            f_aim_p = os.path.join(self.save_path, f)
            if not os.path.exists(f_aim_p):
                f_p = os.path.join(self.root_path, f)
                img = Image.open(f_p)
                # img = img.convert("RGB")
                img = np.array(img, dtype=np.uint8)
                if channel == 4:
                    img = cv2.resize(img, aim_size)[:, :, [2, 1, 0, 3]]
                else:
                    img = cv2.resize(img, aim_size)[:, :, [2, 1, 0]]

                cv2.imwrite(f_aim_p, img)
                # img.save(os.path.join(self.save_path, f))

    def opencv_show(self, ):
        img = cv2.imread(self.img_path)
        cv2.imshow("original", img)
        cv2.waitKey(0)

    def gen_pure_color_img(self, aim_size, color_, save_path):

        img = np.ones((aim_size[0], aim_size[1]), dtype=np.uint8)
        bgr_img = cv2.cvtColor(img, cv2.COLOR_GRAY2BGR)
        # cv2.imshow('bgr_img', bgr_img)
        bgr_img[:, :, 0] = color_[0]
        bgr_img[:, :, 1] = color_[1]
        bgr_img[:, :, 2] = color_[2]
        cv2.imwrite(save_path + '/' + str(color_) + '.png', bgr_img)


if __name__ == '__main__':
    img_p = ''
    root_p = '/workspace/data/waifu_anime/idx33_out_2'
    save_p = '/workspace/data/waifu_anime/idx33_out_2_512'
    img_tools = ImgTools(img_p, root_p, save_p)
    # img_tools.alpha_channel_white()
    # img_tools.convert_2_3channel()
    # img_tools.resize_floder_img(aim_size=(512, 512), channel=3)
    # blue 255,0,0
    img_tools.gen_pure_color_img(aim_size=(512, 512), color_=(0, 0, 255),
                                 save_path='/workspace/codes/stylegan2-ada-pytorch-main/out')



#!/usr/bin/env python
# coding: utf-8

from setuptools import setup, find_packages

setup(
    name='cv2box',  # 项目的名称,pip3 install get-time
    version='0.0.1',  # 项目版本
    author='ykk',  # 项目作者
    author_email='ykk648@gmail.com',  # 作者email
    url='https://github.com/ykk648/cv2box',  # 项目代码仓库
    description='cv toolbox',  # 项目描述
    packages=find_packages(),  # 包名
    install_requires=[],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
)
